import {
  extent,
  scaleLinear,
  scaleTime,
  scaleOrdinal,
  brushX,
} from 'd3';
import { axes } from './axes';
import { colorLegend } from './colorLegend';

export const scatterPlot = (
  selection,
  {
    data,
    width,
    height,
    xValue,
    yValue,
    colorValue,
    margin,
    xAxisLabel,
    yAxisLabel,
    colorLegendLabel,
    colorLegendX,
    colorLegendY,
    setHoveredValue,
    hoveredValue,
    brushInterval,
    setState,
  },
) => {
  const xScale = scaleLinear()
    .domain(extent(data, xValue))
    .range([margin.left, width - margin.right]);

  const yScale = scaleLinear()
    .domain(extent(data, yValue))
    .range([height - margin.bottom, margin.top]);

  selection.call(axes, {
    xScale,
    yScale,
    xAxisLabel,
    yAxisLabel,
  });

  const colorScale = scaleOrdinal()
    .domain(data.map(colorValue))
    .range(['red', 'blue', 'green']);

  selection.call(colorLegend, {
    colorScale,
    colorLegendLabel,
    colorLegendX,
    colorLegendY,
    setHoveredValue,
    hoveredValue,
  });

  selection
    .selectAll('circle.mark')
    .data(data)
    .join('circle')
    .attr('class', 'mark')
    .attr('cx', (d) => xScale(xValue(d)))
    .attr('cy', (d) => yScale(yValue(d)))
    .attr('fill', (d) => colorScale(colorValue(d)))
    .attr('r', 4)
    .attr('stroke', (d) =>
      brushInterval && brushInterval.has(d.id)
        ? 'black'
        : 'none',
    )
    .attr('stroke-width', 5)
    .attr('opacity', (d) =>
      hoveredValue
        ? colorValue(d) === hoveredValue
          ? 1
          : 0.2
        : 0.7,
    );
  //Set brushBebavior
  const brushBehavior = brushX()
    .extent([
      [50, 38],
      [width - 50, height - 51],
    ])
    .on('brush', (event) => {
      //define brush area:
      const [x0, x1] = event.selection;

      const newbrushInterval = new Set();

      for (const d of data) {
        // If d falls within the brushed rectangle
        // then add it to the highlighted set.
        // console.log('xValue:', xValue);
        //const x = column time from data
        // const x = isoParse(d.time);
        //console.log(x);
        // set a cons xValue equals to time from data

        //console.log(xValue);

        const x = xScale(xValue(d));
        if (x >= x0 && x <= x1) {
          newbrushInterval.add(d.id);
        }
      }
      setState((state) => ({
        ...state,
        brushInterval: newbrushInterval,
      }));
    })
    .on('end', (event) => {
      if (event.selection === null) {
        setState((state) => ({
          ...state,
          brushInterval: null,
        }));
      }
    });
  console.log(brushInterval);
  const brushG = selection
    .selectAll('g.brush')
    .data([null])
    .join('g')
    .attr('class', 'brush')
    .call(brushBehavior);
};
